@@ -1,5 +1,5 @@
 /*
- * "$Id: printers.h,v 1.6 2009/10/05 12:37:30 rlk Exp $"
+ * "$Id: printers.h,v 1.10 2015/07/13 02:18:20 speachy Exp $"
  *
  *   libgimpprint printer functions.
  *
@@ -90,6 +90,14 @@ extern const stp_printer_t *stp_get_printer_by_driver(const char *driver);
  */
 extern const stp_printer_t *stp_get_printer_by_device_id(const char *device_id);
 
+/**
+ * Get a printer model by its foomatic ID.
+ * @param foomatic_id the printer model's foomatic ID
+ * @returns a pointer to the printer model, or NULL on failure.  The
+ * pointer should not be freed.
+ */
+extern const stp_printer_t *stp_get_printer_by_foomatic_id(const char *foomatic_id);
+
 /**
  * Get the printer model from a vars object.
  * @param v the vars to use.
@@ -143,6 +151,20 @@ extern const char *stp_printer_get_family(const stp_printer_t *p);
  */
 extern const char *stp_printer_get_manufacturer(const stp_printer_t *p);
 
+/**
+ * Get a printer model's foomatic ID
+ * @param p the printer model to use.
+ * @returns the foomatic ID or NULL (should never be freed)
+ */
+extern const char *stp_printer_get_foomatic_id(const stp_printer_t *p);
+
+/**
+ * Get a printer model's comment string
+ * @param p the printer model to use.
+ * @returns the comments or NULL (should never be freed)
+ */
+extern const char *stp_printer_get_comment(const stp_printer_t *p);
+
 /**
  * Get a printer model's model number.
  * The model number is used internally by the "family" driver module,
@@ -270,5 +292,5 @@ const char *stp_describe_output(const stp_vars_t *v);
 
 #endif /* GUTENPRINT_PRINTERS_H */
 /*
- * End of "$Id: printers.h,v 1.6 2009/10/05 12:37:30 rlk Exp $".
+ * End of "$Id: printers.h,v 1.10 2015/07/13 02:18:20 speachy Exp $".
  */
