@@ -1,7 +1,7 @@
 /*
  *   Canon SELPHY ES/CP series CUPS backend -- libusb-1.0 version
  *
- *   (c) 2007-2014 Solomon Peachy <pizza@shaftnet.org>
+ *   (c) 2007-2015 Solomon Peachy <pizza@shaftnet.org>
  *
  *   The latest version of this program can be found at:
  *
@@ -35,8 +35,12 @@
 #include <fcntl.h>
 #include <signal.h>
 
+#define BACKEND canonselphy_backend
+
 #include "backend_common.h"
 
+#define P_ES40_CP790 (P_END + 1) // used for detection only
+
 /* Exported */
 #define USB_VID_CANON       0x04a9
 #define USB_PID_CANON_CP10  0x304A
@@ -291,13 +295,6 @@ static struct printer_data selphy_printers[] = {
 	  .paper_code_offset = -1,
 	  .error_detect = es3_error_detect,
 	},
-	/* PLACEHOLDER FOR DETECTION PURPOSES ONLY */
-	{ .type = P_ES40_CP790,
-	  .model = "SELPHY ES40/CP790",
-	  .init_length = 16,
-	  .foot_length = 12,
-	  .pgcode_offset = 2,
-	},
 	{ .type = P_ES40,
 	  .model = "SELPHY ES40",
 	  .init_length = 16,
@@ -426,6 +423,7 @@ enum {
 	S_PRINTER_M_SENT,
 	S_PRINTER_READY_C,
 	S_PRINTER_C_SENT,
+	S_PRINTER_CP900_FOOTER,
 	S_PRINTER_DONE,
 	S_FINISHED,
 };
@@ -503,6 +501,7 @@ struct canonselphy_ctx {
 	struct libusb_device_handle *dev;
 	uint8_t endp_up;
 	uint8_t endp_down;
+	int type;
 
 	struct printer_data *printer;
 
@@ -519,27 +518,40 @@ struct canonselphy_ctx {
 	uint8_t *footer;
 
 	uint8_t *buffer;
+
+	uint8_t cp900;
 };
 
 static void *canonselphy_init(void)
 {
 	struct canonselphy_ctx *ctx = malloc(sizeof(struct canonselphy_ctx));
-	if (!ctx)
+	if (!ctx) {
+		ERROR("Memory Allocation Failure!\n");
 		return NULL;
+	}
 	memset(ctx, 0, sizeof(struct canonselphy_ctx));
 
 	/* Static initialization */
 	setup_paper_codes();
 
 	ctx->buffer = malloc(MAX_HEADER);
+	if (!ctx->buffer) {
+		ERROR("Memory Allocation Failure!\n");
+		free(ctx);
+		ctx = NULL;
+	}
 
 	return ctx;
 }
 
+extern struct dyesub_backend canonselphy_backend;
+
 static void canonselphy_attach(void *vctx, struct libusb_device_handle *dev, 
 			       uint8_t endp_up, uint8_t endp_down, uint8_t jobid)
 {
 	struct canonselphy_ctx *ctx = vctx;
+	struct libusb_device *device;
+	struct libusb_device_descriptor desc;
 
 	UNUSED(jobid);
 
@@ -547,28 +559,14 @@ static void canonselphy_attach(void *vctx, struct libusb_device_handle *dev,
 	ctx->endp_up = endp_up;
 	ctx->endp_down = endp_down;
 
-	/* Special cases for some models */
-	if (ctx->printer->type == P_ES40_CP790) {
-		struct libusb_device *device;
-		struct libusb_device_descriptor desc;
-		int i;
-		int printer_type;
+	device = libusb_get_device(dev);
+	libusb_get_device_descriptor(device, &desc);
 
-		device = libusb_get_device(dev);
-		libusb_get_device_descriptor(device, &desc);
-		
-		if (desc.idProduct == USB_PID_CANON_CP790)
-			printer_type = P_CP790;
-		else if (desc.idProduct == USB_PID_CANON_ES40)
-			printer_type = P_ES40;
+	ctx->type = lookup_printer_type(&canonselphy_backend,
+					desc.idVendor, desc.idProduct);
 
-		for (i = 0; selphy_printers[i].type != -1; i++) {
-			if (selphy_printers[i].type == printer_type) {
-				ctx->printer = &selphy_printers[i];
-				break;
-			}
-		}
-	}
+	if (desc.idProduct == USB_PID_CANON_CP900)
+		ctx->cp900 = 1;
 }
 
 static void canonselphy_teardown(void *vctx) {
@@ -610,10 +608,19 @@ static int canonselphy_early_parse(void *vctx, int data_fd)
 		ERROR("Read failed (%d/%d/%d)\n", 
 		      i, 0, MAX_HEADER);
 		perror("ERROR: Read failed");
-		return i;
+		return -1;
 	}
 
 	printer_type = parse_printjob(ctx->buffer, &ctx->bw_mode, &ctx->plane_len);
+	/* Special cases for some models */
+	if (printer_type == P_ES40_CP790) {
+		if (ctx->type == P_CP790)
+			printer_type = P_CP790;
+		else
+			printer_type = P_ES40;			
+	}
+	
+	/* Look up the printer entry */
 	for (i = 0; selphy_printers[i].type != -1; i++) {
 		if (selphy_printers[i].type == printer_type) {
 			ctx->printer = &selphy_printers[i];
@@ -625,14 +632,19 @@ static int canonselphy_early_parse(void *vctx, int data_fd)
 		return -1;
 	}
 
+	INFO("%sFile intended for a '%s' printer\n",  ctx->bw_mode? "B/W " : "", ctx->printer->model);
+
+	if (ctx->printer->type != ctx->type) {
+		ERROR("Printer/Job mismatch (%d/%d)\n", ctx->type, ctx->printer->type);
+		return -1;
+	}
+
 	ctx->plane_len += 12; /* Add in plane header length! */
 	if (ctx->printer->pgcode_offset != -1)
 		ctx->paper_code = ctx->printer->paper_codes[ctx->buffer[ctx->printer->pgcode_offset]];
 	else
 		ctx->paper_code = -1;
 
-	INFO("%sFile intended for a '%s' printer\n",  ctx->bw_mode? "B/W " : "", ctx->printer->model);
-
 	return printer_type;
 }
 
@@ -642,7 +654,12 @@ static int canonselphy_read_parse(void *vctx, int data_fd)
 	int i, remain;
 
 	if (!ctx)
-		return 1;
+		return CUPS_BACKEND_FAILED;
+
+	/* Perform early parsing */
+	i = canonselphy_early_parse(ctx, data_fd);
+	if (i < 0)
+		return CUPS_BACKEND_FAILED;
 
 	if (ctx->header) {
 		free(ctx->header);
@@ -674,7 +691,7 @@ static int canonselphy_read_parse(void *vctx, int data_fd)
 	if (!ctx->plane_y || !ctx->plane_m || !ctx->plane_c || !ctx->header ||
 	    (ctx->printer->foot_length && !ctx->footer)) {
 		ERROR("Memory allocation failure!\n");
-		return 1;
+		return CUPS_BACKEND_FAILED;
 	}
 
 	/* Move over chunks already read in */
@@ -687,7 +704,7 @@ static int canonselphy_read_parse(void *vctx, int data_fd)
 	while (remain > 0) {
 		i = read(data_fd, ctx->plane_y + (ctx->plane_len - remain), remain);
 		if (i < 0)
-			return i;
+			return CUPS_BACKEND_CANCEL;
 		remain -= i;
 	}
 
@@ -696,7 +713,7 @@ static int canonselphy_read_parse(void *vctx, int data_fd)
 	while (remain > 0) {
 		i = read(data_fd, ctx->plane_m + (ctx->plane_len - remain), remain);
 		if (i < 0)
-			return i;
+			return CUPS_BACKEND_CANCEL;
 		remain -= i;
 	}
 
@@ -705,7 +722,7 @@ static int canonselphy_read_parse(void *vctx, int data_fd)
 	while (remain > 0) {
 		i = read(data_fd, ctx->plane_c + (ctx->plane_len - remain), remain);
 		if (i < 0)
-			return i;
+			return CUPS_BACKEND_CANCEL;
 		remain -= i;
 	}
 
@@ -715,12 +732,12 @@ static int canonselphy_read_parse(void *vctx, int data_fd)
 		while (remain > 0) {
 			i = read(data_fd, ctx->footer + (ctx->printer->foot_length - remain), remain);
 			if (i < 0)
-				return i;
+				return CUPS_BACKEND_CANCEL;
 			remain -= i;
 		}
 	}
 
-	return 0;
+	return CUPS_BACKEND_OK;
 }
 
 static int canonselphy_main_loop(void *vctx, int copies) {
@@ -735,7 +752,7 @@ static int canonselphy_main_loop(void *vctx, int copies) {
 			rdbuf, READBACK_LEN, &num);
 
 	if (ret < 0)
-		return ret;
+		return CUPS_BACKEND_FAILED;
 
 top:
 
@@ -748,11 +765,11 @@ top:
 	ret = read_data(ctx->dev, ctx->endp_up,
 			rdbuf, READBACK_LEN, &num);
 	if (ret < 0)
-		return ret;
+		return CUPS_BACKEND_FAILED;
 
 	if (num != READBACK_LEN) {
 		ERROR("Short read! (%d/%d)\n", num, READBACK_LEN);
-		return 4;
+		return CUPS_BACKEND_FAILED;
 	}
 
 	/* Error detection */
@@ -760,8 +777,8 @@ top:
 		if (ctx->printer->clear_error_len)
 			/* Try to clear error state */
 			if ((ret = send_data(ctx->dev, ctx->endp_down, ctx->printer->clear_error, ctx->printer->clear_error_len)))
-				return ret;
-		return 4;
+				return CUPS_BACKEND_FAILED;
+		return CUPS_BACKEND_HOLD;
 	}
 
 	if (memcmp(rdbuf, rdbuf2, READBACK_LEN)) {
@@ -781,26 +798,26 @@ top:
 		
 		/* Make sure paper/ribbon is correct */
 		if (ctx->paper_code != -1) {
-			if (ctx->printer->type == P_CP_XXX) {
+			if (ctx->type == P_CP_XXX) {
 				uint8_t pc = rdbuf[ctx->printer->paper_code_offset];
 				if (((pc >> 4) & 0xf) != (ctx->paper_code & 0x0f)) {
 
 					if (pc & 0xf0) {
 						ERROR("Incorrect paper tray loaded, aborting job!\n");
-						return 3;
+						return CUPS_BACKEND_HOLD;
 					} else {
 						ERROR("No paper tray loaded, aborting!\n");
-						return 4;
+						return CUPS_BACKEND_STOP;
 					}
 				}
 				if ((pc & 0xf) != (ctx->paper_code & 0xf)) {
 					if (pc & 0x0f) {
 						ERROR("Incorrect ribbon loaded, aborting job!\n");
-						return 3;
+						return CUPS_BACKEND_HOLD;
 					} else {
 
 						ERROR("No ribbon loaded, aborting job!\n");
-						return 4;
+						return CUPS_BACKEND_STOP;
 					}
 				}
 			} else {
@@ -809,26 +826,26 @@ top:
 					ERROR("Incorrect media/ribbon loaded (%02x vs %02x), aborting job!\n", 
 					      ctx->paper_code,
 					      rdbuf[ctx->printer->paper_code_offset]);
-					return 3;  /* Hold this job, don't stop queue */
+					return CUPS_BACKEND_HOLD;  /* Hold this job, don't stop queue */
 				}
 			}
-		} else if (ctx->printer->type == P_CP790) {
+		} else if (ctx->type == P_CP790) {
 			uint8_t ribbon = rdbuf[4] >> 4;
 			uint8_t paper = rdbuf[5];
 
 			if (ribbon == 0xf) {
 				ERROR("No ribbon loaded, aborting!\n");
-				return 4;	
+				return CUPS_BACKEND_STOP;	
 			} else if (ribbon != ctx->paper_code) {
 				ERROR("Incorrect ribbon loaded, aborting job!\n");
-				return 3;
+				return CUPS_BACKEND_HOLD;
 			}
 			if (paper == 0xf) {
 				ERROR("No paper tray loaded, aborting!\n");
-				return 4;
+				return CUPS_BACKEND_STOP;
 			} else if (paper != ctx->paper_code) {
 				ERROR("Incorrect paper loaded, aborting job!\n");
-				return 3;
+				return CUPS_BACKEND_HOLD;
 			}
 		}
 
@@ -838,7 +855,7 @@ top:
 		INFO("Printing started; Sending init sequence\n");
 		/* Send printer init */
 		if ((ret = send_data(ctx->dev, ctx->endp_down, ctx->header, ctx->printer->init_length)))
-			return ret;
+			return CUPS_BACKEND_FAILED;
 
 		state = S_PRINTER_INIT_SENT;
 		break;
@@ -854,7 +871,7 @@ top:
 			INFO("Sending YELLOW plane\n");
 
 		if ((ret = send_data(ctx->dev, ctx->endp_down, ctx->plane_y, ctx->plane_len)))
-			return ret;
+			return CUPS_BACKEND_FAILED;
 
 		state = S_PRINTER_Y_SENT;
 		break;
@@ -870,7 +887,7 @@ top:
 		INFO("Sending MAGENTA plane\n");
 
 		if ((ret = send_data(ctx->dev, ctx->endp_down, ctx->plane_m, ctx->plane_len)))
-			return ret;
+			return CUPS_BACKEND_FAILED;
 
 		state = S_PRINTER_M_SENT;
 		break;
@@ -883,21 +900,35 @@ top:
 		INFO("Sending CYAN plane\n");
 
 		if ((ret = send_data(ctx->dev, ctx->endp_down, ctx->plane_c, ctx->plane_len)))
-			return ret;
+			return CUPS_BACKEND_FAILED;
 
 		state = S_PRINTER_C_SENT;
 		break;
 	case S_PRINTER_C_SENT:
 		if (!fancy_memcmp(rdbuf, ctx->printer->done_c_readback, READBACK_LEN)) {
-			state = S_PRINTER_DONE;
+			if (ctx->cp900)
+				state = S_PRINTER_CP900_FOOTER;
+			else 
+				state = S_PRINTER_DONE;
 		}
 		break;
+	case S_PRINTER_CP900_FOOTER: {
+		uint32_t empty = 0;
+
+		INFO("Sending CP900 Footer\n");
+		if ((ret = send_data(ctx->dev, ctx->endp_down, 
+				     (uint8_t*)&empty, sizeof(empty))))
+			return CUPS_BACKEND_FAILED;
+
+		state = S_PRINTER_DONE;
+		break;
+	}
 	case S_PRINTER_DONE:
 		if (ctx->printer->foot_length) {
 			INFO("Cleaning up\n");
 
 			if ((ret = send_data(ctx->dev, ctx->endp_down, ctx->footer, ctx->printer->foot_length)))
-				return ret;
+				return CUPS_BACKEND_FAILED;
 		}
 		state = S_FINISHED;
 		/* Intentional Fallthrough */
@@ -919,17 +950,39 @@ top:
 		goto top;
 	}
 
+	return CUPS_BACKEND_OK;
+}
+
+static int canonselphy_cmdline_arg(void *vctx, int argc, char **argv)
+{
+	struct canonselphy_ctx *ctx = vctx;
+	int i, j = 0;
+
+	if (!ctx)
+		return -1;
+	
+	/* Reset arg parsing */
+	optind = 1;
+	opterr = 0;
+	while ((i = getopt(argc, argv, GETOPT_LIST_GLOBAL)) >= 0) {
+		switch(i) {
+		GETOPT_PROCESS_GLOBAL
+		}
+
+		if (j) return j;
+	}
+
 	return 0;
 }
 
 struct dyesub_backend canonselphy_backend = {
 	.name = "Canon SELPHY CP/ES",
-	.version = "0.82.2G",
+	.version = "0.89",
 	.uri_prefix = "canonselphy",
+	.cmdline_arg = canonselphy_cmdline_arg,
 	.init = canonselphy_init,
 	.attach = canonselphy_attach,
 	.teardown = canonselphy_teardown,
-	.early_parse = canonselphy_early_parse,
 	.read_parse = canonselphy_read_parse,
 	.main_loop = canonselphy_main_loop,
 	.devices = {
@@ -953,7 +1006,7 @@ struct dyesub_backend canonselphy_backend = {
 	{ USB_VID_CANON, USB_PID_CANON_CP760, P_CP_XXX, ""},
 	{ USB_VID_CANON, USB_PID_CANON_CP770, P_CP_XXX, ""},
 	{ USB_VID_CANON, USB_PID_CANON_CP780, P_CP_XXX, ""},
-	{ USB_VID_CANON, USB_PID_CANON_CP790, P_ES40_CP790, ""},
+	{ USB_VID_CANON, USB_PID_CANON_CP790, P_CP790, ""},
 	{ USB_VID_CANON, USB_PID_CANON_CP800, P_CP_XXX, ""},
 	{ USB_VID_CANON, USB_PID_CANON_CP810, P_CP_XXX, ""},
 	{ USB_VID_CANON, USB_PID_CANON_CP900, P_CP_XXX, ""},
@@ -962,7 +1015,7 @@ struct dyesub_backend canonselphy_backend = {
 	{ USB_VID_CANON, USB_PID_CANON_ES20, P_ES2_20, ""},
 	{ USB_VID_CANON, USB_PID_CANON_ES3, P_ES3_30, ""},
 	{ USB_VID_CANON, USB_PID_CANON_ES30, P_ES3_30, ""},
-	{ USB_VID_CANON, USB_PID_CANON_ES40, P_ES40_CP790, ""},
+	{ USB_VID_CANON, USB_PID_CANON_ES40, P_ES40, ""},
 	{ 0, 0, 0, ""}
 	}
 };
@@ -1284,6 +1337,8 @@ struct dyesub_backend canonselphy_backend = {
   adding a 50x50mm sticker and 22x17.3mm ministickers, though I think the
   driver treats all of those as 'C' sizes for printing purposes.
 
+  Printer does *not* apparently require use of a spooler!
+
   32-byte header:
 
   0f 00 00 40 00 00 00 00  00 00 00 00 00 00 01 00
@@ -1305,8 +1360,8 @@ struct dyesub_backend canonselphy_backend = {
         05  (L)
         02  (C)
 
-  P == 7008800  == 2336256 * 3 + 32 (4.884% larger than CP)
-  L == 5087264  == 1695744 * 3 + 32 (5.878% larger than CP)
-  C == 2180384  == 726784 * 3 + 32  (3.991% larger than CP)
+  P == 7008800  == 2336256 * 3 + 32 (1872*1248)
+  L == 5087264  == 1695744 * 3 + 32 (1536*1104)
+  C == 2180384  == 726784 * 3 + 32  (1088*668)
 
 */
