@@ -1,5 +1,5 @@
 /*
- * "$Id: dither-ordered.c,v 1.28 2008/02/18 14:20:17 rlk Exp $"
+ * "$Id: dither-ordered.c,v 1.31 2010/08/04 00:33:56 rlk Exp $"
  *
  *   Ordered dither algorithm
  *
@@ -33,7 +33,6 @@
 #include <gutenprint/gutenprint-intl-internal.h>
 #include "dither-impl.h"
 #include "dither-inlined-functions.h"
-#include <assert.h>
 
 
 typedef struct {
@@ -180,11 +179,6 @@ print_color_ordered_new(const stpi_dither_t *d, stpi_dither_channel_t *dc,
   unsigned short *where = ord ? ord->lut + (val * levels) : &swhere;
   /*
    * Look for the appropriate range into which the input value falls.
-   * Notice that we use the input, not the error, to decide what dot type
-   * to print (if any).  We actually use the "density" input to permit
-   * the caller to use something other that simply the input value, if it's
-   * desired to use some function of overall density, rather than just
-   * this color's input, for this purpose.
    */
   for (i = levels - 1; i >= 0; i--)
     {
@@ -223,11 +217,6 @@ print_color_ordered(const stpi_dither_t *d, stpi_dither_channel_t *dc, int val,
 
   /*
    * Look for the appropriate range into which the input value falls.
-   * Notice that we use the input, not the error, to decide what dot type
-   * to print (if any).  We actually use the "density" input to permit
-   * the caller to use something other that simply the input value, if it's
-   * desired to use some function of overall density, rather than just
-   * this color's input, for this purpose.
    */
   for (i = levels; i >= 0; i--)
     {
@@ -274,7 +263,7 @@ free_dither_ordered(stpi_dither_t *d)
   int i;
   stpi_dither_channel_t *dc0 = &CHANNEL(d, 0);
   stpi_ordered_t *o0 = dc0->aux_data;
-  stpi_new_ordered_t *no0;
+  stpi_new_ordered_t *no0 = NULL;
   if (o0)
     no0 = o0->ord_new;
   for (i = CHANNEL_COUNT(d) - 1; i >= 0 ; i--)
