@@ -1,5 +1,5 @@
 /*
- * "$Id: print-escp2.h,v 1.132 2008/07/11 01:24:25 rlk Exp $"
+ * "$Id: print-escp2.h,v 1.140 2014/01/23 13:22:41 rlk Exp $"
  *
  *   Print plug-in EPSON ESC/P2 driver for the GIMP.
  *
@@ -28,8 +28,8 @@
  * Maximum number of channels in a printer.  If Epson comes out with an
  * 8-head printer, this needs to be increased.
  */
-#define PHYSICAL_CHANNEL_LIMIT 8
 #define MAX_DROP_SIZES 3
+#define PHYSICAL_CHANNEL_LIMIT ((STP_CHANNEL_LIMIT) / ((MAX_DROP_SIZES)+1))
 
 #define XCOLOR_R     (STP_NCOLORS + 0)
 #define XCOLOR_B     (STP_NCOLORS + 1)
@@ -161,6 +161,8 @@ typedef struct
   const char *subchannel_transition;
   const char *subchannel_value;
   const char *subchannel_scale;
+  const char *name;
+  const char *text;
   short *split_channels;
 } physical_subchannel_t;
 
@@ -180,8 +182,9 @@ typedef enum
   INKSET_CcMmYyK          = 2,
   INKSET_CcMmYKk          = 3,
   INKSET_QUADTONE         = 4,
-  INKSET_OTHER		  = 5,
-  INKSET_EXTENDED	  = 6
+  INKSET_HEXTONE          = 5,
+  INKSET_OTHER		  = 6,
+  INKSET_EXTENDED	  = 7
 } inkset_id_t;
 
 typedef struct
@@ -191,6 +194,8 @@ typedef struct
   short channel_count;
   short aux_channel_count;
   inkset_id_t inkset;
+  const stp_raw_t *init_sequence;
+  const stp_raw_t *deinit_sequence;
   ink_channel_t *channels;
   ink_channel_t *aux_channels;
 } inkname_t;
@@ -207,6 +212,8 @@ typedef struct
   const char *text;
   short n_shades;
   short n_inks;
+  const stp_raw_t *init_sequence;
+  const stp_raw_t *deinit_sequence;
   shade_t *shades;
   inkname_t *inknames;
 } inklist_t;
@@ -241,6 +248,7 @@ typedef struct
   short is_cd;
   short is_roll_feed;
   short duplex;
+  short extra_height;
   unsigned roll_feed_cut_flags;
   const stp_raw_t *init_sequence;
   const stp_raw_t *deinit_sequence;
@@ -260,43 +268,44 @@ typedef struct
 #define MODEL_COMMAND_2000	0x2ul /* ESC i printers with extended ESC(c */
 #define MODEL_COMMAND_PRO	0x3ul /* Stylus Pro printers */
 
-#define MODEL_ZEROMARGIN_MASK	0x30ul /* Does this printer support */
+#define MODEL_ZEROMARGIN_MASK	0x70ul /* Does this printer support */
 #define MODEL_ZEROMARGIN_NO	0x00ul /* zero margin mode? */
 #define MODEL_ZEROMARGIN_YES	0x10ul /* (print beyond bottom of page) */
 #define MODEL_ZEROMARGIN_FULL	0x20ul /* (do not print beyond bottom) */
-#define MODEL_ZEROMARGIN_H_ONLY	0x30ul /* (no special treatment for vertical) */
+#define MODEL_ZEROMARGIN_RESTR	0x30ul /* (no special treatment for vertical) */
+#define MODEL_ZEROMARGIN_H_ONLY	0x40ul /* (borderless only horizontal) */
 
-#define MODEL_VARIABLE_DOT_MASK	0x40ul /* Does this printer support var */
+#define MODEL_VARIABLE_DOT_MASK	0x80ul /* Does this printer support var */
 #define MODEL_VARIABLE_NO	0x00ul /* dot size printing? The newest */
-#define MODEL_VARIABLE_YES	0x40ul /* printers support multiple modes */
+#define MODEL_VARIABLE_YES	0x80ul /* printers support multiple modes */
 
-#define MODEL_GRAYMODE_MASK	0x80ul /* Does this printer support special */
-#define MODEL_GRAYMODE_NO	0x00ul /* fast black printing? */
-#define MODEL_GRAYMODE_YES	0x80ul
+#define MODEL_GRAYMODE_MASK	0x100ul /* Does this printer support special */
+#define MODEL_GRAYMODE_NO	0x000ul /* fast black printing? */
+#define MODEL_GRAYMODE_YES	0x100ul
 
-#define MODEL_FAST_360_MASK	0x100ul
+#define MODEL_FAST_360_MASK	0x200ul
 #define MODEL_FAST_360_NO	0x000ul
-#define MODEL_FAST_360_YES	0x100ul
+#define MODEL_FAST_360_YES	0x200ul
 
-#define MODEL_SEND_ZERO_ADVANCE_MASK	0x200ul
+#define MODEL_SEND_ZERO_ADVANCE_MASK	0x400ul
 #define MODEL_SEND_ZERO_ADVANCE_NO	0x000ul
-#define MODEL_SEND_ZERO_ADVANCE_YES	0x200ul
+#define MODEL_SEND_ZERO_ADVANCE_YES	0x400ul
 
-#define MODEL_SUPPORTS_INK_CHANGE_MASK	0x400ul
+#define MODEL_SUPPORTS_INK_CHANGE_MASK	0x800ul
 #define MODEL_SUPPORTS_INK_CHANGE_NO	0x000ul
-#define MODEL_SUPPORTS_INK_CHANGE_YES	0x400ul
+#define MODEL_SUPPORTS_INK_CHANGE_YES	0x800ul
 
-#define MODEL_PACKET_MODE_MASK	0x800ul
-#define MODEL_PACKET_MODE_NO	0x000ul
-#define MODEL_PACKET_MODE_YES	0x800ul
+#define MODEL_PACKET_MODE_MASK	0x1000ul
+#define MODEL_PACKET_MODE_NO	0x0000ul
+#define MODEL_PACKET_MODE_YES	0x1000ul
 
-#define MODEL_INTERCHANGEABLE_INK_MASK	0x1000ul
+#define MODEL_INTERCHANGEABLE_INK_MASK	0x2000ul
 #define MODEL_INTERCHANGEABLE_INK_NO	0x0000ul
-#define MODEL_INTERCHANGEABLE_INK_YES	0x1000ul
+#define MODEL_INTERCHANGEABLE_INK_YES	0x2000ul
 
-#define MODEL_ENVELOPE_LANDSCAPE_MASK	0x2000ul
+#define MODEL_ENVELOPE_LANDSCAPE_MASK	0x4000ul
 #define MODEL_ENVELOPE_LANDSCAPE_NO	0x0000ul
-#define MODEL_ENVELOPE_LANDSCAPE_YES	0x2000ul
+#define MODEL_ENVELOPE_LANDSCAPE_YES	0x4000ul
 
 typedef enum
 {
@@ -401,6 +410,11 @@ typedef struct escp2_printer
   short		m_roll_right_margin;	/* Right margin, points */
   short		m_roll_top_margin;	/* Absolute top margin, points */
   short		m_roll_bottom_margin;	/* Absolute bottom margin, points */
+				/* Duplex margin limit (SHRT_MIN = no limit): */
+  short		duplex_left_margin;	/* Left margin, points */
+  short		duplex_right_margin;	/* Right margin, points */
+  short		duplex_top_margin;	/* Absolute top margin, points */
+  short		duplex_bottom_margin;	/* Absolute bottom margin, points */
 				/* Print directly to CD */
   short		cd_x_offset;	/* Center of CD (horizontal offset) */
   short		cd_y_offset;	/* Center of CD (vertical offset) */
@@ -550,6 +564,7 @@ typedef struct
   const res_t *res;		/* Description of the printing resolution */
   const stp_raw_t *printer_weave; /* Printer weave parameters */
   int use_printer_weave;	/* Use the printer weaving mechanism */
+  int extra_vertical_passes;	/* Quality enhancement */
 
   /* page parameters */		/* Indexed from top left */
   int page_left;		/* Left edge of page (points) */
@@ -609,5 +624,5 @@ extern void stpi_escp2_terminate_page(stp_vars_t *v);
 
 #endif /* GUTENPRINT_INTERNAL_ESCP2_H */
 /*
- * End of "$Id: print-escp2.h,v 1.132 2008/07/11 01:24:25 rlk Exp $".
+ * End of "$Id: print-escp2.h,v 1.140 2014/01/23 13:22:41 rlk Exp $".
  */
