@@ -1,5 +1,5 @@
 /*
- * "$Id: print-util.c,v 1.117 2010/06/26 20:02:02 rlk Exp $"
+ * "$Id: print-util.c,v 1.122 2015/09/09 23:57:32 speachy Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -64,9 +64,9 @@ typedef struct
 
 #define STPI_VASPRINTF(result, bytes, format)				\
 {									\
-  int current_allocation = 64;						\
+  int current_allocation = 64; /* vsnprintf returns int */		\
   result = stp_malloc(current_allocation);				\
-  while (1)								\
+  while (current_allocation < INT_MAX / 2)				\
     {									\
       va_list args;							\
       va_start(args, format);						\
@@ -639,8 +639,8 @@ stp_merge_printvars(stp_vars_t *user, const stp_vars_t *print)
 stp_parameter_list_t
 stp_get_parameter_list(const stp_vars_t *v)
 {
-  stp_parameter_list_t ret = stp_parameter_list_create();
-  stp_parameter_list_t tmp_list;
+  stp_parameter_list_t *ret = stp_parameter_list_create();
+  stp_parameter_list_t *tmp_list;
 
   tmp_list = stp_printer_list_parameters(v);
   stp_parameter_list_append(ret, tmp_list);
@@ -650,10 +650,6 @@ stp_get_parameter_list(const stp_vars_t *v)
   stp_parameter_list_append(ret, tmp_list);
   stp_parameter_list_destroy(tmp_list);
 
-  tmp_list = stp_dither_list_parameters(v);
-  stp_parameter_list_append(ret, tmp_list);
-  stp_parameter_list_destroy(tmp_list);
-
   tmp_list = stp_list_generic_parameters(v);
   stp_parameter_list_append(ret, tmp_list);
   stp_parameter_list_destroy(tmp_list);
@@ -664,5 +660,10 @@ stp_get_parameter_list(const stp_vars_t *v)
 void
 stp_abort(void)
 {
+  if (stp_get_debug_level() > 0)
+    {
+      int i = *((volatile int *) 0);
+      stp_erprintf("Crashing here...%d\n", i);
+    }
   abort();
 }
