@@ -1,5 +1,5 @@
 /*
- * "$Id: printers.c,v 1.85 2008/06/03 11:40:39 rlk Exp $"
+ * "$Id: printers.c,v 1.94 2015/07/13 02:18:20 speachy Exp $"
  *
  *   Print plug-in driver utility functions for the GIMP.
  *
@@ -65,6 +65,9 @@ struct stp_printer
   char       *long_name;        /* Long name for UI */
   char       *family;           /* Printer family */
   char	     *manufacturer;	/* Printer manufacturer */
+  char	     *device_id; 	/* IEEE 1284 device ID */
+  char       *foomatic_id;	/* Foomatic printer ID */
+  char       *comment;	     	/* Comment string, if any */
   int        model;             /* Model number */
   int	     vars_initialized;
   const stp_printfuncs_t *printfuncs;
@@ -108,20 +111,6 @@ stp_printer_model_count(void)
   return stp_list_get_length(printer_list);
 }
 
-static void
-null_printer(void)
-{
-  stp_erprintf("Null stp_printer_t! Please report this bug.\n");
-  stp_abort();
-}
-
-static inline void
-check_printer(const stp_printer_t *p)
-{
-  if (p == NULL)
-    null_printer();
-}
-
 const stp_printer_t *
 stp_get_printer_by_index(int idx)
 {
@@ -142,6 +131,16 @@ static void
 stpi_printer_freefunc(void *item)
 {
   stp_printer_t *printer = (stp_printer_t *) item;
+  if (printer->comment)
+    {
+      stp_free(printer->comment);
+      printer->comment = NULL;
+    }
+  if (printer->foomatic_id)
+    {
+      stp_free(printer->foomatic_id);
+      printer->comment = NULL;
+    }
   stp_free(printer->long_name);
   stp_free(printer->family);
   stp_free(printer);
@@ -186,6 +185,12 @@ stpi_printer_long_namefunc(const void *item)
   return printer->long_name;
 }
 
+const char *
+stp_printer_get_device_id(const stp_printer_t *printer)
+{
+  return printer->device_id;
+}
+
 const char *
 stp_printer_get_family(const stp_printer_t *printer)
 {
@@ -198,6 +203,18 @@ stp_printer_get_manufacturer(const stp_printer_t *printer)
   return printer->manufacturer;
 }
 
+const char *
+stp_printer_get_foomatic_id(const stp_printer_t *printer)
+{
+  return printer->foomatic_id;
+}
+
+const char *
+stp_printer_get_comment(const stp_printer_t *printer)
+{
+  return printer->comment;
+}
+
 int
 stp_printer_get_model(const stp_printer_t *printer)
 {
@@ -243,6 +260,54 @@ stp_get_printer_by_driver(const char *driver)
   return (const stp_printer_t *) stp_list_item_get_data(printer_item);
 }
 
+const stp_printer_t *
+stp_get_printer_by_device_id(const char *device_id)
+{
+  stp_list_item_t *printer_item;
+  if (printer_list == NULL)
+    {
+      stp_erprintf("No printer drivers found: "
+		   "are STP_DATA_PATH and STP_MODULE_PATH correct?\n");
+      stpi_init_printer_list();
+    }
+  if (! device_id || strcmp(device_id, "") == 0)
+    return NULL;
+
+  printer_item = stp_list_get_start(printer_list);
+  while (printer_item)
+    {
+      if (strcmp(((const stp_printer_t *) stp_list_item_get_data(printer_item))->device_id,
+		 device_id) == 0)
+	return ((const stp_printer_t *) stp_list_item_get_data(printer_item));
+      printer_item = stp_list_item_next(printer_item);
+    }
+  return NULL;
+}
+
+const stp_printer_t *
+stp_get_printer_by_foomatic_id(const char *foomatic_id)
+{
+  stp_list_item_t *printer_item;
+  if (printer_list == NULL)
+    {
+      stp_erprintf("No printer drivers found: "
+		   "are STP_DATA_PATH and STP_MODULE_PATH correct?\n");
+      stpi_init_printer_list();
+    }
+  if (! foomatic_id || strcmp(foomatic_id, "") == 0)
+    return NULL;
+
+  printer_item = stp_list_get_start(printer_list);
+  while (printer_item)
+    {
+      if (strcmp(((const stp_printer_t *) stp_list_item_get_data(printer_item))->foomatic_id,
+		 foomatic_id) == 0)
+	return ((const stp_printer_t *) stp_list_item_get_data(printer_item));
+      printer_item = stp_list_item_next(printer_item);
+    }
+  return NULL;
+}
+
 int
 stp_get_printer_index_by_driver(const char *driver)
 {
@@ -401,7 +466,7 @@ stp_printer_get_defaults(const stp_printer_t *printer)
 {
   if (! printer->vars_initialized)
     {
-      stp_printer_t *nc_printer = (stp_printer_t *) printer;
+      stp_printer_t *nc_printer = (stp_printer_t *) stpi_cast_safe(printer);
       stp_deprintf(STP_DBG_PRINTERS, "  ==>init %s\n", printer->driver);
       set_printer_defaults (nc_printer->printvars, 1, 0);
       nc_printer->vars_initialized = 1;
@@ -934,6 +999,9 @@ stp_family_register(stp_list_t *family)
 	  printer = (const stp_printer_t *) stp_list_item_get_data(printer_item);
 	  if (!stp_list_get_item_by_name(printer_list, printer->driver))
 	    stp_list_item_create(printer_list, NULL, printer);
+	  else
+	    stp_erprintf("Duplicate printer entry `%s' (%s)\n",
+			 printer->driver, printer->long_name);
 	  printer_item = stp_list_item_next(printer_item);
 	}
     }
@@ -1024,8 +1092,10 @@ stp_printer_create_from_xmltree(stp_mxml_node_t *printer, /* The printer node */
                                                        /* Family printfuncs */
 {
   stp_mxml_node_t *prop;	/* Temporary node pointer */
+  stp_mxml_node_t *child;
   const char *stmp;		/* Temporary string */
   stp_printer_t *outprinter;	/* Generated printer */
+  size_t slen = 0;
   int
     driver = 0,			/* Check driver */
     long_name = 0;
@@ -1054,6 +1124,39 @@ stp_printer_create_from_xmltree(stp_mxml_node_t *printer, /* The printer node */
   outprinter->manufacturer = stp_strdup(stp_mxmlElementGetAttr(printer, "manufacturer"));
   outprinter->model = stp_xmlstrtol(stp_mxmlElementGetAttr(printer, "model"));
   outprinter->family = stp_strdup((const char *) family);
+  stmp = stp_mxmlElementGetAttr(printer, "deviceid");
+  if (stmp)
+    outprinter->device_id = stp_strdup(stmp);
+  stmp = stp_mxmlElementGetAttr(printer, "foomaticid");
+  if (stmp)
+    outprinter->foomatic_id = stp_strdup(stmp);
+
+  child = printer->child;
+  while (child)
+    {
+      if (child->type == STP_MXML_TEXT)
+	{
+	  if (outprinter->comment)
+	    {
+	      size_t oslen = slen;
+	      slen += strlen(child->value.text.string);
+	      if (child->value.text.whitespace)
+		slen += 1;
+	      outprinter->comment = stp_realloc(outprinter->comment, slen + 1);
+	      (void) memset(outprinter->comment + oslen, 0, slen - oslen);
+	      if (child->value.text.whitespace)
+		  outprinter->comment[oslen++] = ' ';
+	      strncat(outprinter->comment + oslen,
+		      child->value.text.string, slen - oslen);
+	    }
+	  else
+	    {
+	      outprinter->comment = stp_strdup(child->value.text.string);
+	      slen = strlen(outprinter->comment);
+	    }
+	}
+      child = child->next;
+    }
 
   if (stp_get_driver(outprinter->printvars))
     driver = 1;
